"""
Helper for our tests
"""

# Django
from django.core.handlers.wsgi import WSGIRequest
from django.template import Context, Template


def render_template(string, context=None):
    """
    Helper to render templates
    :param string:
    :param context:
    :return:
    """

    context = context or {}
    context = Context(dict_=context)

    return Template(template_string=string).render(context=context)


def response_content_to_str(response: WSGIRequest) -> str:
    """
    Return the content of a WSGIRequest response as string

    :param response:
    :type response:
    :return:
    :rtype:
    """

    return response.content.decode(response.charset)



# Django
from django.conf import settings
from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings

# AA Mumble Quick Connect
from aa_mumble_quick_connect.models import MumbleLink, Section, validate_mumble_url


class TestSection(TestCase):
    """
    Tests for the `Section` model
    """

    def test_model_string_names(self):
        """
        Test model string name

        :return:
        :rtype:
        """

        section = Section(name="Foobar")
        section.save()
        expected_name = section.name
        self.assertEqual(first=str(section), second=expected_name)


class TestMumbleLink(TestCase):
    """
    Tests for the `MumbleLink` model
    """

    def test_model_string_names(self):
        """
        Test model string name

        :return:
        :rtype:
        """

        section = MumbleLink(name="Foobar")
        section.save()
        expected_name = section.name
        self.assertEqual(first=str(section), second=expected_name)

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_validate_mumble_url_with_valid_url(self):
        """
        Test validate Mumble URL with valid URL

        :return:
        :rtype:
        """

        expected_mumble_base_url = f"mumble://{settings.MUMBLE_URL}"

        self.assertEqual(
            first=validate_mumble_url(url=f"mumble://{settings.MUMBLE_URL}"),
            second=expected_mumble_base_url,
        )

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_validate_mumble_url_with_invalid_url(self):
        """
        Test validate Mumble URL with invalid URL raises ValidationError

        :return:
        :rtype:
        """

        expected_mumble_base_url = f"mumble://{settings.MUMBLE_URL}"

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message=f'The Mumble channel URL must start with "{expected_mumble_base_url}"',
        ):
            validate_mumble_url("mumble://example.com")

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_validate_mumble_url_with_disabled_url_verification(self):
        """
        Test validate Mumble URL with disabled URL verification

        :return:
        :rtype:
        """

        self.assertEqual(
            first=validate_mumble_url(
                url="mumble://example.com", disable_url_verification=True
            ),
            second="mumble://example.com",
        )

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_validate_mumble_url_with_invalid_url_protocol(self):
        """
        Test validate Mumble URL with invalid URL protocol raises ValidationError

        :return:
        :rtype:
        """

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message='The Mumble channel URL must start with "mumble://"',
        ):
            validate_mumble_url(url="https://example.com")

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_validate_mumble_url_with_invalid_url_protocol_and_disabled_url_verification(
        self,
    ):
        """
        Test validate Mumble URL with invalid URL protocol and disabled URL verification raises ValidationError

        :return:
        :rtype:
        """

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message='The Mumble channel URL must start with "mumble://"',
        ):
            validate_mumble_url(
                url="https://example.com", disable_url_verification=True
            )

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_save_with_valid_url(self):
        """
        Test save with valid URL

        :return:
        :rtype:
        """

        mumble_link = MumbleLink(
            name="Foobar",
            url="mumble://mumble.example.com",
            section=Section.objects.create(name="Foobar"),
        )
        mumble_link.save()

        self.assertEqual(first=mumble_link.url, second="mumble://mumble.example.com")

    @override_settings(MUMBLE_URL="mumble.example.com")
    def test_save_with_invalid_url_raises_validation_error(self):
        """
        Test save with invalid URL raises ValidationError

        :return:
        :rtype:
        """

        mumble_link = MumbleLink(
            name="Foobar",
            url="mumble://example.com",
            section=Section.objects.create(name="Foobar"),
        )

        expected_mumble_base_url = f"mumble://{settings.MUMBLE_URL}"

        with self.assertRaisesMessage(
            expected_exception=ValidationError,
            expected_message=f'The Mumble channel URL must start with "{expected_mumble_base_url}"',
        ):
            mumble_link.clean()

    @override_settings(MUMBLE_URL="mumble.example.com")
    def tes_save_with_invalid_url_does_not_raise_validation_error(self):
        """
        Test save with invalid URL does not raise ValidationError

        :return:
        :rtype:
        """

        mumble_link = MumbleLink(
            name="Foobar",
            url="mumble://example.com",
            section=Section.objects.create(name="Foobar"),
            disable_url_verification=True,
        )

        mumble_link.clean()
        mumble_link.save()

        self.assertEqual(first=mumble_link.url, second="mumble://example.com")



"""
App Configuration
"""

# Django
from django.apps import AppConfig

# AA Mumble Quick Connect
from aa_mumble_quick_connect import __version__


class AaMumbleQuickConnectConfig(AppConfig):
    """
    App configuration
    """

    name = "aa_mumble_quick_connect"
    label = "aa_mumble_quick_connect"
    verbose_name = f"Mumble Quick Connect v{__version__}"



"""
Test auth_hooks
"""

# Standard Library
from http import HTTPStatus

# Django
from django.test import TestCase, modify_settings
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_fake_user

# AA Mumble Quick Connect
from aa_mumble_quick_connect.tests.utils import response_content_to_str


class TestAccess(TestCase):
    """
    Test access
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        # User
        cls.user_1001 = create_fake_user(
            character_id=1001,
            character_name="Jean Luc Picard",
            permissions=[
                "aa_mumble_quick_connect.basic_access",
                "mumble.access_mumble",
            ],
        )

        cls.user_1002 = create_fake_user(
            character_id=1002,
            character_name="William Riker",
            permissions=["mumble.access_mumble"],
        )

        cls.user_1003 = create_fake_user(
            character_id=1003,
            character_name="Worf",
            permissions=["aa_mumble_quick_connect.basic_access"],
        )

        cls.user_1004 = create_fake_user(
            character_id=1004, character_name="Wesley Crusher"
        )

        cls.html_menu = f"""
            <li class="d-flex flex-wrap m-2 p-2 pt-0 pb-0 mt-0 mb-0 me-0 pe-0">
                <i class="nav-link fa-solid fa-headphones-simple fa-fw align-self-center me-3 active"></i>
                <a class="nav-link flex-fill align-self-center me-auto active" href="{reverse('aa_mumble_quick_connect:index')}">
                    Mumble Quick Connect
                </a>
            </li>
        """

        cls.header_top = '<div class="navbar-brand">Mumble Quick Connect</div>'

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.mumble"})
    def test_access_for_user_with_permission(self):
        """
        Test access for user with permission

        :return:
        :rtype:
        """

        self.client.force_login(user=self.user_1001)

        response = self.client.get(
            path=reverse(viewname="aa_mumble_quick_connect:index")
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.OK)
        self.assertInHTML(
            needle=self.html_menu, haystack=response_content_to_str(response)
        )
        self.assertInHTML(
            needle=self.header_top, haystack=response_content_to_str(response)
        )

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.mumble"})
    def test_access_for_user_with_just_mumble_permission(self):
        """
        Test access for user with just mumble permission

        :return:
        :rtype:
        """

        self.client.force_login(user=self.user_1002)

        response = self.client.get(
            path=reverse(viewname="aa_mumble_quick_connect:index")
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.FOUND)
        self.assertNotIn(
            member=self.html_menu, container=response_content_to_str(response)
        )
        self.assertNotIn(
            member=self.header_top, container=response_content_to_str(response)
        )

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.mumble"})
    def test_access_for_user_with_just_aa_mumble_quick_connect_permission(self):
        """
        Test access for user with just aa_mumble_quick_connect permission

        :return:
        :rtype:
        """

        self.client.force_login(user=self.user_1003)

        response = self.client.get(
            path=reverse(viewname="aa_mumble_quick_connect:index")
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.FOUND)
        self.assertNotIn(
            member=self.html_menu, container=response_content_to_str(response)
        )
        self.assertNotIn(
            member=self.header_top, container=response_content_to_str(response)
        )

    @modify_settings(INSTALLED_APPS={"append": "allianceauth.services.modules.mumble"})
    def test_access_for_user_without_permission(self):
        """
        Test access for user without permission

        :return:
        :rtype:
        """

        self.client.force_login(user=self.user_1004)

        response = self.client.get(
            path=reverse(viewname="aa_mumble_quick_connect:index")
        )

        self.assertEqual(first=response.status_code, second=HTTPStatus.FOUND)
        self.assertNotIn(
            member=self.html_menu, container=response_content_to_str(response)
        )
        self.assertNotIn(
            member=self.header_top, container=response_content_to_str(response)
        )



"""
Hook into Alliance Auth
"""

# Django
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook

# AA Mumble Quick Connect
from aa_mumble_quick_connect import urls
from aa_mumble_quick_connect.dependency_checks import mumble_service_installed


class AaMumbleQuickConnectMenuItem(MenuItemHook):
    """
    This class ensures only authorized users will see the menu entry
    """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self=self,
            text=str(_("Mumble Quick Connect")),
            classes="fa-solid fa-headphones-simple",
            url_name="aa_mumble_quick_connect:index",
            navactive=["aa_mumble_quick_connect:"],
        )

    def render(self, request):
        """
        Render the menu item
        """

        if not mumble_service_installed():
            return ""

        if request.user.has_perm(
            "aa_mumble_quick_connect.basic_access"
        ) and request.user.has_perm("mumble.access_mumble"):
            return MenuItemHook.render(self=self, request=request)

        return ""


@hooks.register("menu_item_hook")
def register_menu():
    """
    Register the menu item
    """

    return AaMumbleQuickConnectMenuItem()


@hooks.register("url_hook")
def register_urls():
    """
    Register app urls
    """

    return UrlHook(
        urls=urls,
        namespace="aa_mumble_quick_connect",
        base_url=r"^mumble-quick-connect/",
    )



# Generated by Django 4.2.19 on 2025-02-13 12:30

# Django
import django.db.models.deletion
from django.db import migrations, models

# AA Mumble Quick Connect
import aa_mumble_quick_connect.models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (("basic_access", "Can access this app"),),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Section",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Name of the section", max_length=255, unique=True
                    ),
                ),
            ],
            options={
                "verbose_name": "Section",
                "verbose_name_plural": "Sections",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="MumbleLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Name of the Mumble channel", max_length=255
                    ),
                ),
                (
                    "url",
                    models.CharField(
                        help_text="URL to the channel",
                        max_length=255,
                        validators=[aa_mumble_quick_connect.models.validate_mumble_url],
                    ),
                ),
                (
                    "section",
                    models.ForeignKey(
                        blank=True,
                        help_text="Section the Mumble channel belongs to. (Optional)",
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="mumble_links",
                        to="aa_mumble_quick_connect.section",
                        verbose_name="Section",
                    ),
                ),
            ],
            options={
                "verbose_name": "Mumble Link",
                "verbose_name_plural": "Mumble Links",
                "default_permissions": (),
            },
        ),
    ]
